# Code written by Andrea Nova. University of Pavia.

sharedenvironmentmatrix <- function (id_fam, parentsrel1,parentsrel2, siblingsrel, grandparentsrel, unclesrel,cousinsrel,spousesrel,n.cores2)
{
  
  if (!require("mbend")) {
    install.packages("mbend")
  }
  
  if (!require("FamAgg")) {
    install.packages("FamAgg")
  }
  
  if (!require("matrixcalc")) {
    install.packages("matrixcalc")
  }
  
  if (!require("parallel")) {
    install.packages("parallel")
  }
  
  if (!require("corpcor")) {
    install.packages("corpcor")
  }
  
  library(mbend)
  library(FamAgg)
  library(matrixcalc)
  library(parallel)
  library(corpcor)
  
  "%nin%"<-Negate("%in%")
  getCousins<-function(ped,id){
    parents <-  getAncestors(ped, id = id,max.generations = 1)
    grandparents <-  getAncestors(ped, id =parents,max.generations = 1)
    grandparents<-grandparents[grandparents!="0"]
    uncles <- getChildren(ped, id = grandparents, max.generations = 1)
    cousins <- getChildren(ped, id = uncles, max.generations = 1)
    cousins
  }
  
  getUncles<-function(ped,id){
    parents <-  getAncestors(ped, id = id,max.generations = 1)
    grandparents <-  getAncestors(ped, id =parents,max.generations = 1)
    grandparents<-grandparents[grandparents!="0"]
    uncles <- getChildren(ped, id = grandparents, max.generations = 1)
    uncles
  }
  
  colnames(id_fam) <- c("family", "id", "father", "mother", "sex")
  id_fam$sex[id_fam$sex==1]<-2
  id_fam$sex[id_fam$sex==0]<-1
  
  famlist <- unique(id_fam$family)
  nfam <- length(famlist)
  matlist <- vector("list", nfam)
  genH <- function(i,id_fam, parentsrel1,parentsrel2, siblingsrel, grandparentsrel, unclesrel,cousinsrel,spousesrel){
    family <- id_fam[which(id_fam$family==famlist[i]),]
    
    temp<-diag(1,length(family$id))
    rownames(temp)<-colnames(temp)<-as.character(family$id)
    
    for (j in 1:length(family$id)) {
      fat<-NULL
      mot<-NULL
      parents<-getAncestors(family, id=family$id[j], max.generations=1)
      parents<-parents[parents!="0"]
      
      if(siblingsrel>0){
        fad <- suppressMessages(FAData(id_fam))
        siblings<-getSiblings(fad, id=family$id[j])
      }
      if(grandparentsrel>0){
        grandparents<-getAncestors(family, id=family$id[j], max.generations=2)
        grandparents<-grandparents[grandparents!="0"]
        grandparents<-grandparents[grandparents %nin% parents]
      }
      if(unclesrel>0){
        uncles<-getUncles(family, id=family$id[j])
        uncles<-uncles[which(uncles %nin% parents)]
      }
      if(cousinsrel>0){
        cousins<-getCousins(family, id=family$id[j])
        cousins<-cousins[cousins %nin% siblings]
      }
      children<-getChildren(family, id=family$id[j], max.generations=1)
      mother<-parents[which(parents %in% family$mother)]
      father<-parents[which(parents %in% family$father)]
      
      if(length(parents)>0){
        temp[as.character(family$id[j]),as.character(mother)]<-parentsrel1
        temp[as.character(mother),as.character(family$id[j])]<-parentsrel1
        temp[as.character(family$id[j]),as.character(father)]<-parentsrel2
        temp[as.character(father),as.character(family$id[j])]<-parentsrel2
      }
      if(siblingsrel>0){
        siblings<-siblings[siblings %nin% family$id[j]]
        temp[as.character(family$id[j]),as.character(siblings)]<-siblingsrel
        temp[as.character(siblings),as.character(family$id[j])]<-siblingsrel
      }
      if(grandparentsrel>0){
        temp[as.character(family$id[j]),as.character(grandparents)]<-grandparentsrel
        temp[as.character(grandparents),as.character(family$id[j])]<-grandparentsrel
      }
      if(unclesrel>0){
        temp[as.character(family$id[j]),as.character(uncles)]<-unclesrel
        temp[as.character(uncles),as.character(family$id[j])]<-unclesrel
      }
      if(cousinsrel>0){
        cousins<-cousins[cousins!=family$id[j]]
        temp[as.character(family$id[j]),as.character(cousins)]<-cousinsrel
        temp[as.character(cousins),as.character(family$id[j])]<-cousinsrel
      }
    }
    
    if(spousesrel>0){
      iddd<-data.frame(id=family$id, dad=family$father, 
                       mom=family$mother, sex=family$sex, famid=family$family)
      tped <- with(iddd, kinship2::pedigree(id, dad, mom, sex, famid=famid, missid=0 ))
      nonfounders <- (tped$mindex+tped$findex)>0
      spouses <- cbind(tped$mindex, tped$findex)[nonfounders,]
      
      
      
      if(length(spouses)==2){
        temp[cbind(spouses[1],spouses[2])] <- spousesrel
        temp[cbind(spouses[2],spouses[1])] <- spousesrel
      }
      
      if(length(spouses)>2){
        temp[spouses] <- spousesrel
        temp[spouses[,2:1]] <- spousesrel
      }
    }
    matlist<- as(Matrix::forceSymmetric(temp), "dsCMatrix")
    return(matlist)
  }
  
  matlist2 <- mclapply(1:length(famlist),genH,id_fam, parentsrel1,parentsrel2,siblingsrel, grandparentsrel, unclesrel,cousinsrel,spousesrel,mc.cores=n.cores2)
  
  result <- Matrix::bdiag(matlist2)
  
  if (any(duplicated(id_fam$id))) {
    dimnames(result) <- list(NULL, paste(id_fam$famid, id_fam$id,
                                         sep = "/"))
  } else { dimnames(result) <- list(id_fam$id, id_fam$id) }
  result
}
