functions {
//This function defines the log-likelihood function for the multivariate normal distribution.

real partial_sum(int[] fam_slice,
                   int start, int end,
                 matrix X, matrix XP,vector YP, int[] ni, int[] firstfam, vector lb, vector ub, vector lb_ind, vector ub_ind, vector u, int NFAM, real heritability, real sharedenvironment_sibs, real individualenvironment, matrix kinship_matrix,matrix sharedenvironment_matrix_sibs,matrix identity_matrix, vector betapam,int NCOV, real threshold) {

real lik=0;
 for (k in start:end) {
int nik =ni[k];
int firstfamk =firstfam[k];
row_vector[NCOV] XPk =XP[k,];
real YPk =YP[k];

// Covariance matrix. You can update it to include/remove variance components.
matrix [nik,nik] Sigmakc=cholesky_decompose(block(kinship_matrix, firstfamk, firstfamk, nik, nik)*heritability + block(sharedenvironment_matrix_sibs, firstfamk, firstfamk, nik, nik)*sharedenvironment_sibs+  block(identity_matrix, firstfamk, firstfamk, nik, nik)*(individualenvironment));

matrix[nik,NCOV] Xk=block(X, firstfamk ,1,nik,NCOV);
vector[nik] lbk=segment(lb, firstfamk ,nik);
vector[nik] ubk=segment(ub, firstfamk ,nik);
vector[nik] lb_indk=segment(lb_ind, firstfamk ,nik);
vector[nik] ub_indk=segment(ub_ind, firstfamk ,nik);
vector[nik] uk=segment(u, firstfamk ,nik);

vector[nik] mu = Xk*betapam;

// To correct for ascertainment bias
real XB = XPk*betapam;
real alpha=YPk*log((1-normal_cdf(threshold - XB, 0, 1))/(1-(1-normal_cdf(threshold - XB, 0, 1)))) - log(1/(1-(1-normal_cdf(threshold - XB, 0, 1))));

   vector[nik] z;
     real prob = 0;
      for ( m in 1:nik ) {
        if ( lb_indk[m] == 0 && ub_indk[m] == 0 )  z[m] = inv_Phi(uk[m]);
          else { 
            int km1 = m - 1;
            real v; 
            real z_star;
            real logd;
            row_vector [2] log_ustar = [negative_infinity(), 0];           
            real constrain = mu[m] + ((m > 1) ? Sigmakc[m, 1:km1] * head(z, km1) : 0);
            if ( lb_indk[m] == 1 ) log_ustar[1] = normal_lcdf( ( lbk[m] - constrain ) / Sigmakc[m, m] | 0.0, 1.0 );
            if ( ub_indk[m] == 1 ) log_ustar[2] = normal_lcdf( ( ubk[m] - constrain ) / Sigmakc[m, m] | 0.0, 1.0 );
            logd  = log_diff_exp(log_ustar[2], log_ustar[1]);
            v     = exp( log_sum_exp( log_ustar[1], log(uk[m]) + logd ) );    
            z[m]  = inv_Phi(v);                                          
            prob  += logd;                                                    
          }
        }
          lik += prob-alpha;
}
   return  lik; 
  }
}

data {
//Definition of data as defined in R code template.

int<lower=0> N; 
matrix[N,N] kinship_matrix;
matrix[N,N] sharedenvironment_matrix_sibs;
matrix[N,N] identity_matrix; 
real threshold; 
vector[N] lb; 
vector[N] ub; 
vector<lower=0,upper=1>[N] lb_ind;
vector<lower=0,upper=1>[N] ub_ind;
int<lower=0> NFAM;
int ni[NFAM];
int firstfam[NFAM];
int<lower=0> NCOV;
matrix[N,NCOV] X;
vector[NCOV] SD;
matrix[NFAM,NCOV] XP;
vector[NFAM] YP;
int<lower=1> fam[NFAM];
int<lower=1> grainsize;
}


parameters {
//Define parameters.

vector[NCOV] betapam;
real <lower=0, upper= 1> sharedenvironment_sibs;
real <lower=0, upper= 1-sharedenvironment_sibs> heritability;
vector<lower=0,upper=1>[N] u; //Latent parameters for the liability scores.
}

transformed parameters{
// Individual environment derived form sum constraint.

real <lower=0, upper= 1> individualenvironment = 1-heritability-sharedenvironment_sibs;
}

model {
// Change the prior distirbutions if needed
betapam~ normal(0,10);
heritability~ beta(1,1);
sharedenvironment_sibs~ beta(1,1);

target += reduce_sum(partial_sum,fam , grainsize,X, XP, YP,ni, firstfam, lb, ub, lb_ind, ub_ind, u, NFAM,heritability,sharedenvironment_sibs, individualenvironment, kinship_matrix,sharedenvironment_matrix_sibs,identity_matrix, betapam,NCOV,threshold);

    }
    
generated quantities{
real sigmaCOV=variance(X*betapam);

//Unstandardized fixed effects
real beta_COV = betapam[1] / SD[1];

//Marginal parameters posterior distributions
real <lower=0, upper= 1> heritability_marginal = heritability/(1+sigmaCOV);
real <lower=0, upper= 1> sharedenvironment_sibs_marginal = sharedenvironment_sibs/(1+sigmaCOV);
real <lower=0, upper= 1> individualenvironment_marginal = individualenvironment/(1+sigmaCOV);
real <lower=0, upper= 1> tauCOV_marginal = sigmaCOV/(1+sigmaCOV);

}
